package com.perfectech.tis.syncClasses;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.IBinder;
import android.location.Location;
import android.os.Bundle;
import android.util.Log;

import java.util.ArrayList;
import java.util.Date;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DBParameters;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.JobClass;
import com.perfectech.tis.objects.Jobs;
import com.perfectech.tis.objects.JobTrackingClass;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.*;
import com.google.android.gms.location.LocationListener;

import org.json.JSONObject;


public class JobTracking extends Service implements GoogleApiClient.ConnectionCallbacks,
        GoogleApiClient.OnConnectionFailedListener,
        LocationListener {

    /********************************************************************************/
    /* Define Global Variables */
    /**
     * ****************************************************************************
     */

    private static final String sDBName = "TIS.db";
    private static ArrayList<ArrayList<String>> jobArray;
    static int iVersionVal = 6;
    private GoogleApiClient googleApiClient;
    private LocationRequest mLocationRequest = new LocationRequest();
    public static long UPDATE_INTERVAL_IN_MILLISECONDS = 300000;
    public static long FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS =
            UPDATE_INTERVAL_IN_MILLISECONDS / 2;
    private static boolean debug_mode = false;
    private static long SYNC_DELAY_MINS=0;
    private static long GPS_GATHER_MINS = 1;
    private Date oTimeofLastPoll = new Date();
    private Date oTimeofLastUpload = new Date();
    private Boolean bFirstPoll = true;
    private String sUserID = "";
    private Location lastLocation = null;
    private float fDistance = 0;

    @Override
    public void onLocationChanged(Location location) {
        final Location oLocation = location;
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Location mCurrentLocation = oLocation;
                    DateConvertClass oDateConvertClass = new DateConvertClass();
                    oDateConvertClass.oContext = getApplicationContext();
                    String sCurrentDateTime = oDateConvertClass.GMTTimestamp();

                    DBParameters oDBParameters = new DBParameters();
                    try {
                        String sDebugMode = oDBParameters.readParameter("DebugMode",
                                getApplicationContext(), sDBName, iVersionVal);
                        if (sDebugMode.equals("1")) {
                            debug_mode = true;
                        } else {
                            debug_mode = false;
                        }
                    } catch (Exception ex) {
                        DBLogs.insert("DB Error: " + ex.toString(),
                                "JobTracking.onLocationChanged()",
                                getApplicationContext(), sDBName, iVersionVal);
                    }

                    if (debug_mode == true) {
                        DBLogs.insert("Sync DateTime: " + sCurrentDateTime +
                                        " GPS_POLL_MINS: " + String.valueOf((UPDATE_INTERVAL_IN_MILLISECONDS / 60) / 1000) + " Minutes",
                                "JobTracking.onLocationChanged()",
                                getApplicationContext(), sDBName, iVersionVal);
                        DBLogs.insert("Sync DateTime: " + sCurrentDateTime +
                                        " SYNC_DELAY_MINS: " + String.valueOf((SYNC_DELAY_MINS / 60) / 1000) + " Minutes",
                                "JobTracking.onLocationChanged()",
                                getApplicationContext(), sDBName, iVersionVal);
                    }

                    // get a list of jobs
                    JobClass jobClass = new JobClass();
                    ArrayList<Jobs> jobs = new ArrayList<Jobs>();
                    boolean bUpdated = false;
                    boolean bTimeForNewPoll = false;
                    boolean bUpdateLastPoll = false;
                    boolean bSaveGPSDataToDB = false;

                    jobClass.oContext = getApplicationContext();
                    jobs = jobClass.read();

                    ArrayList<String> alCoords = new ArrayList<String>();
                    String sTime = getGMTDataTime();
                    float fAccuracy = mCurrentLocation.getAccuracy();
                    float fSpeed = mCurrentLocation.getSpeed();

                    // check to see if time for new poll
                    Date oNow = new Date();
                    long oSaveLocalDiff = oNow.getTime() - oTimeofLastPoll.getTime();
                    long oDiff = oNow.getTime() - oTimeofLastUpload.getTime();
                    if (oSaveLocalDiff > GPS_GATHER_MINS) {
                        bSaveGPSDataToDB = true;
                        oTimeofLastPoll = new Date();
                    } else {
                        bSaveGPSDataToDB = false;
                    }

                    if (oDiff > SYNC_DELAY_MINS) {
                        bTimeForNewPoll = true;
                        oTimeofLastUpload = new Date();
                    } else {
                        bTimeForNewPoll = false;
                    }
                    if (!jobs.isEmpty()) {
                        for (Jobs job : jobs) {
                            try {
                                Double dLat = mCurrentLocation.getLatitude();
                                Double dLong = mCurrentLocation.getLongitude();
                                if (dLat != 0.0 && dLong != 0.0) {
                                    JSONObject oCoord = new JSONObject();
                                    oCoord.put("JobID", job.job_id);
                                    oCoord.put("Lat", String.valueOf(dLat));
                                    oCoord.put("Long", String.valueOf(dLong));
                                    oCoord.put("Time", sTime);
                                    oCoord.put("userID", sUserID);
                                    oCoord.put("Accuracy", Float.valueOf(fAccuracy));
                                    oCoord.put("Speed", Float.valueOf(fSpeed));
                                    oCoord.put("Distance", Float.valueOf(fDistance));
                                    oCoord.put("Type", "job");
                                    alCoords.add(oCoord.toString());
                                    Log.d("Job Number", job.job_num);
                                    Log.d("Job ID", job.job_id);

                                }
                                // First check to see if there are any geofences crossed.
                                // Will use same bUpdated variable until it is used for something
                                SynJobTracking.oContext = getApplicationContext();
                                bUpdated = SynJobTracking.testForGEOFence(job, mCurrentLocation);
                                if (debug_mode == true) {
                                    if (bUpdated == false) {
                                        DBLogs.insert("Geofence area notice NOT sent.",
                                                "JobTracking.onLocationChanged().isOnline",
                                                getApplicationContext(), sDBName, iVersionVal);
                                    } else {
                                        DBLogs.insert("Geofence area notice sent.",
                                                "JobTracking.onLocationChanged().isOnline",
                                                getApplicationContext(), sDBName, iVersionVal);
                                    }
                                }

                            } catch (Exception ex) {
                                ex.printStackTrace();
                                DBLogs.insert("Error.SynJobTracking - Sync DateTime: " + sCurrentDateTime,
                                        ex.toString(),
                                        getApplicationContext(), sDBName, iVersionVal);
                            }
                        }

                        // need to see if it is time to place GPS coords into the local database
                        if (bUpdated || bSaveGPSDataToDB || bFirstPoll) {
                            if (!alCoords.isEmpty()) {
                                SynJobTracking.oContext = getApplicationContext();
                                SynJobTracking.saveLocationDataToDatabase(alCoords);
                            }
                            bFirstPoll = false;
                        }

                        // need to see if it is time to send GPS coords into the server
                        if (bTimeForNewPoll) {
                            if (TISCommon.isOnline(getApplicationContext())) {
                                if (debug_mode == true) {
                                    DBLogs.insert("JobTracking.isOnLine(): We are online need to sync job data",
                                            "JobTracking.onLocationChanged()",
                                            getApplicationContext(), sDBName, iVersionVal);
                                }
                                SynJobTracking.sendJobCoords();
                            } else {
                                DBLogs.insert("JobTracking.isOnLine(): We are not online and can not sync job data",
                                        "JobTracking.onLocationChanged()",
                                        getApplicationContext(), sDBName, iVersionVal);
                            }
                        }
                    }

                    lastLocation = mCurrentLocation;
                } catch (Exception ex) {
                    ex.printStackTrace();
                    DBLogs.insert("Error: " + ex.toString(),
                            "JobTracking.onLocationChange.Thread()",
                            getApplicationContext(), sDBName, iVersionVal);
                }
            }
        }).start();
    }

    //GoogleApiClient
    @Override
    public void onConnectionFailed(ConnectionResult bundle) {
        DBLogs.insert("Error: " + bundle.getErrorCode() + " " + bundle.toString(),
                "JobTracking.onConnectionFailed()",
                getApplicationContext(), sDBName, iVersionVal);
    }

    @Override
    public void onConnected(Bundle bundle) {
        Log.i("onConnected", "GoogleApiClient");
        try {
            createLocationRequest();
            LocationServices.FusedLocationApi.requestLocationUpdates(
                    googleApiClient, mLocationRequest, this);
            LocationServices.FusedLocationApi.getLastLocation(googleApiClient);

        } catch (Throwable t) { //you should always ultimately catch all exceptions in timer tasks.
            Log.e("Google APi Connected", "Google APi Connected Failed.", t);
            DBLogs.insert("Error: " + t.toString(),
                    "JobTracking.onConnected()",
                    getApplicationContext(), sDBName, iVersionVal);
        }

    }

    @Override
    public void onConnectionSuspended(int i) {

    }

    @Override
    public IBinder onBind(Intent intent) {
        // TODO: Return the communication channel to the service.
        throw new UnsupportedOperationException("Not yet implemented");
    }

    @Override
    public void onCreate() {
        super.onCreate();
        if (debug_mode == true) {
            DBLogs.insert("Starting up background sync service", "JobTracking.onCreate",
                    getApplicationContext(), sDBName, iVersionVal);
        }

        try {
            googleApiClient = new GoogleApiClient.Builder(this)

                    .addApi(LocationServices.API)
                    .addConnectionCallbacks(this)
                    .addOnConnectionFailedListener(this)

                    .build();

            googleApiClient.connect();

        } catch (Exception e) {
            e.printStackTrace();
            DBLogs.insert("Error: " + e.toString(),
                    "JobTracking.onCreate()",

                    getApplicationContext(), sDBName, iVersionVal);
        } finally {
        }

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        sUserID = prefs.getString("UserID", "No Value");

        _startService();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        //Stop location services on service destroy
        googleApiClient.disconnect();
        _shutdownService();
    }

    private void _startService() {

        // need to get the number of minutes for the sync delay ans gps poll mins
        DBParameters oDBParameters = new DBParameters();

        String sSyncDelayMins = "";
        String sGPSGatherMins = "";

        try {
            sSyncDelayMins = oDBParameters.readParameter("SYNC_JOB_DELAY_MINS",
                    getApplicationContext(), sDBName, iVersionVal);
            sGPSGatherMins = oDBParameters.readParameter("GPS_POLL_MINS",
                    getApplicationContext(), sDBName, iVersionVal);
        } catch (Exception e) {
            DBLogs.insert("Error: " + e.toString(),
                    "JobTracking._startService()",
                    getApplicationContext(), sDBName, iVersionVal);
        }

        if (sSyncDelayMins.length() < 1) {
            sSyncDelayMins = "5";
        }

        if (sGPSGatherMins.length() < 1) {
            sGPSGatherMins = "1";
        }

        SYNC_DELAY_MINS = (Long.parseLong(sSyncDelayMins) * 60) * 1000;
        //This is every 15 seconds
        UPDATE_INTERVAL_IN_MILLISECONDS = (60 / 4)  * 1000;
        FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS = UPDATE_INTERVAL_IN_MILLISECONDS;
        //We poll the UPDATE_INTERVAL_IN_MILLISECONDS for location mostly to test for goefence
        //We will store if geofence is crossed or every GPS_GATHER_MINS
        GPS_GATHER_MINS = (Long.parseLong(sGPSGatherMins) * 60) * 1000;
    }

    protected void createLocationRequest() {
        mLocationRequest.setInterval(UPDATE_INTERVAL_IN_MILLISECONDS);
        mLocationRequest.setFastestInterval(FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS);
        mLocationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
    }

    private boolean isGooglePlayServicesAvailable() {
        int status = GooglePlayServicesUtil.isGooglePlayServicesAvailable(this);
        if (ConnectionResult.SUCCESS == status) {
            return true;
        } else {
            //GooglePlayServicesUtil.getErrorDialog(status, this, 0).show();
            return false;
        }
    }

    private void _shutdownService() {
        //if (timer != null)
        //    timer.cancel();
    }

    private void createJobTrackingRec(Jobs jobsClass, Double dLat, Double dLong) {
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        String sCurrentDateTime = oDateConvertClass.GMTTimestamp();
        JobTrackingClass oJobTrackingClass = new JobTrackingClass();
        oJobTrackingClass.jobStatus = jobsClass.job_status_id;
        oJobTrackingClass.jobNum = jobsClass.job_num;
        oJobTrackingClass.jobNote = "Automated updated from Smartphone with current GPS information.";
        oJobTrackingClass.locid1 = dLat.toString();
        oJobTrackingClass.logid2 = dLong.toString();
        oJobTrackingClass.userid = jobsClass.username;
        oJobTrackingClass.date_time = sCurrentDateTime;
    }

    // Get the GMT time that will be inserted into any log messages
    private String getGMTDataTime() {
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();;
        return oDateConvertClass.GMTTimestamp();
    }
}
